/* $Id: NajaRef.hh,v 1.1 2002/04/27 12:59:33 sdevaux Exp $
 *
 * NaJA
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */ 
#ifndef __NajaRef_hh__
#define __NajaRef_hh__

PACKAGE_BEGIN(najaLang)

class Object;	

/**
 * This template handle a java-like reference. It hides the pointer
 * and calls use() and unuse() when necessary. Mainly when assigning
 * references.<br>
 * Warning : naja::lang::Object must be a super class of _naja
 * qui hrite de Object.
 */
template <class _naja> class NAJA{
private:	
	/** pointer to the object handled by this NAJA ref */
	Object * ptr;
public:
	/**
	 * Create a new naja reference
	 * @param obj pointer to the object to handle
	 */ 
	NAJA(_naja * obj=NULL) {
		MSTART("NAJA::NAJA(_naja*)")
		ptr=(Object *)obj;
		if (ptr!=NULL)
			ptr->use();
		MEND
	};
	/**
	 * Create a new naja reference
	 * @param obj the object to handle
	 */ 
	NAJA(_naja& obj) {
		MSTART("NAJA::NAJA(_naja&)")
		ptr=(Object *)&obj;
		if (ptr!=NULL)
			ptr->use();
		MEND
	};
	/**
	 * copy constructor
	 * @param a naja reference to copy
	 */
	NAJA(const NAJA& a) {
		MSTART("NAJA::NAJA(NAJA&)")
		ptr=a.ptr;ptr->use();
		MEND
	};
	/**
	 * When the reference destroyed, the handled Object is no more used thrue
	 * this reference
	 */
	~NAJA() {
		MSTART("NAJA::~NAJA()")
		if (ptr!=NULL) {
			ptr->unuse();
		}
		MEND
	};
	/**
	 * assign a reference
	 * @param a reference to assign
	 */
	NAJA<_naja>& operator=(const NAJA& a) {
		MSTART("NAJA::operator=(NAJA&)")
		if (this!=&a) {
        	if (ptr!=NULL) ptr->unuse();
        	ptr=(Object *)a.ptr;
        	ptr->use();
    	}
    	MRETURN(*this)
	};
	/**
	 * assign an object to this reference
	 * @param a pointer to the object to assign
	 */
	NAJA<_naja>& operator=(_naja * a) {
		MSTART("NAJA::operator=(_naja*)")
		if (ptr!=(Object *)a) {
        	if (ptr!=NULL)
            	ptr->unuse();
        	ptr=(Object *)a;
        	ptr->use();
    	}
    	MRETURN(*this)
	};
	/**
	 * assign an object to this reference
	 * @param a the object to assign
	 */
	NAJA<_naja>& operator=(_naja& a) {
		MSTART("NAJA::operator=(_naja&)")
		if (ptr!=(Object *)&a) {
			if (ptr!=NULL)
				ptr->unuse();
			ptr=(Object *)&a;
			ptr->use();
		}
		MRETURN(*this)
	};
	/**
	 * let handled object properties and methods available directly
	 * thrue naja reference.
	 */
	inline _naja* operator->() {
		MSTART("NAJA::operator->()")
		MRETURN((_naja*)ptr)
	};
	/**
	 * a naja reference is a rich pointer to the handled object.
	 */
	inline _naja& operator*() {
		MSTART("NAJA::operator*()")
		MRETURN((_naja&)*ptr)
	};
	/**
	 * NAJA refs are equals (== not .equals() !) when pointed objects
	 * are equals (== not .equals() !)
	 */
	inline int operator==(const NAJA& n) {
		return this->ptr==n.ptr;
	}
	inline int operator==(const _naja& a) {
		return ptr==(Object *)&a;
	}
	/**
	 * Surcharge deplacement dans tableau
	 */
//	_naja& operator[](int i) {return ptr[i];};
};	

template <class _naja> ostream& operator<<(ostream& s,NAJA<_naja>& n) {
	return s << n->toString()->c_str();
};

PACKAGE_END
#endif // __NajaRef_hh__
