/* $Id: Object.hh,v 1.1 2002/04/27 12:59:33 sdevaux Exp $
 *
 * NaJA
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */ 
#ifndef __lang_Object_hh__
#define __lang_Object_hh__

PACKAGE_BEGIN(najaLang)
	
class String;
/**
 * TODO mutex to protect static var access (and may be useCount) 
 */
class Object {
private:
	/** The next object to destroy */
	static Object * nextToDelete;
	/**
	 * a naja object can be destroy when its use count reach 0. Then
	 * it becomes "destroyable".
	 * An object can't destroy itself, so this static method just
	 * remember the address of the unused object to delete it later.
     */
	static void free(Object * toDel);
	/** usage count */
	int useCount;
public:
	/**
	 * Default constructor, makes a new void object
     */
	Object();
	/**
	 * Destructor
	 */
	 virtual ~Object();
    /**
	 * Called exactly when useCount reach 0. Since a delay can occure
	 * before real destruction. Just in Time clean up should be handle
	 * here.
     */
	virtual void finalize() {};
	/**
	 * notify the object is used once again
	 */
	void use();
	/**
     * notify someone is no more using the object
	 */
    void unuse();
	/**
	 * A default hash code is available for all objects : the address
	 * @return a hash code value for this object.
	 */
	virtual int hashCode();
	/**
	 * Indicates whether some other object is "equal to" this one. 
     * The equals method implements an equivalence relation:<ul> 
     *    <li>It is reflexive: for any reference value x, x.equals(x) should 
	 *		  return true.</li> 
     *    <li>It is symmetric: for any reference values x and y, x.equals(y) 
	 *	      should return true if and only if y.equals(x)
     *        returns true.</li> 
     *    <li>It is transitive: for any reference values x, y, and z, if x.equals(y)
	 *		  returns true and y.equals(z) returns
     *        true, then x.equals(z) should return true.</li> 
     *    <li>It is consistent: for any reference values x and y, multiple
	 *        invocations of x.equals(y) consistently return true or
     *        consistently return false, provided no information used in equals 
	 *        comparisons on the object is modified.</li> 
     *    <li>For any non-null reference value x, x.equals(null) should return false.</li> 
	 * </ul>
     * The equals method for class Object implements the most discriminating possible
	 * equivalence relation on objects; that is,
     * for any reference values x and y, this method returns true if and only if x and
	 * y refer to the same object (x==y has the
     * value true).
	 * @param obj the reference object with which to compare.
	 * @return true if this object is the same as the obj argument; false otherwise.
	 */ 
	virtual boolean equals(NAJA<Object> obj);
	/**
	 * @return a textual representation of the object
	 */
	virtual NAJA<String> toString();
	/**
     * copy the object
	 * @return a new object, it should be equals to this object
	 */
	virtual NAJA<Object> clone();
	/**
	 * equals operator is different to equals method !
	 */
	virtual inline int operator==(const Object& o) {
		return this==&o;
	};
	/**
	 * Default ostream conversion is textual representation
	 */
	friend ostream& operator<<(ostream& s,Object* o);
	friend ostream& operator<<(ostream& s,Object& o);
};

extern "C++" {
ostream& operator<<(ostream& s,Object* o);
ostream& operator<<(ostream& s,Object& o);
}

PACKAGE_END

#endif // __Object_hh__
