/* $Id: Object.cc,v 1.1 2002/04/27 12:59:33 sdevaux Exp $
 *
 * NaJA
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */ 

#include "naja.hh"

#include "lang/String.hh"

PACKAGE_BEGIN(najaLang)
//-------------------------------------------------------------------
// when starting, nothing to destroy.
Object * Object::nextToDelete=NULL;
//-------------------------------------------------------------------
// freeing an object : it can now be destroyed anytime
void Object::free(Object * toDel) {
	SMSTART("Object::free(@"<<toDel<<")")
	if (nextToDelete!=NULL) {
		DBUG("deleteting @" << nextToDelete)
		delete nextToDelete;
	}	
	DBUG("to delete @" << toDel)
	nextToDelete=toDel;
	MEND
}	
//-------------------------------------------------------------------
// default constructor
 Object::Object() {
	MSTART("Object::Object()")
	useCount=0;
	MEND
}
//-------------------------------------------------------------------
// Destructor
Object::~Object() {
	MSTART("Object::~JObject()")
	if (useCount!=0) {
		TRACE("This object (@"<<this<<") should not be destoyed !!")
	}		
	MEND
}		
//-------------------------------------------------------------------
// using this object
void Object::use() {
	MSTART("Object::use()")
	useCount++;
	DBUG("count=" << useCount)
	MEND
}
//-------------------------------------------------------------------
// no more using this object
void Object::unuse() {
	MSTART("Object::unuse()")
	useCount--;
	DBUG("count=" << useCount)
	if (useCount==0) {
		this->finalize();
		Object::free(this);
	}	
	MEND
}	
//-------------------------------------------------------------------
// default hash code is object's address
int Object::hashCode() {
	MSTART("Object::hashCode()")
	MRETURN((int)this)
}
//-------------------------------------------------------------------
// default equality is identity (same address)
boolean Object::equals(NAJA<Object> obj) {
	MSTART("Object::equals("<<obj<<")")
	MRETURN(this==&(*obj))
}
//-------------------------------------------------------------------
// default string repersentation is address in textual form.
NAJA<String> Object::toString() {
	MSTART("Object::toString()")
	char buf[255];
	sprintf(buf,"%p",this);
	MRETURN(new String(buf))
}
//-------------------------------------------------------------------
// Default clone is return a new void object
NAJA<Object> Object::clone() {
	MSTART("Object::clone")
	MRETURN(new Object())
}
//-------------------------------------------------------------------
// ostream convert
ostream& operator<<(ostream& s,Object* o) {
	return s << o->toString()->c_str();
}
ostream& operator<<(ostream& s,Object& o) {
	return s << o.toString()->c_str();
}

PACKAGE_END
